using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.IO;
using System.Text;
using System.Reflection;
 
namespace OlSentItemsEvents
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("174F5135-D822-435F-A316-9E9353F4662E"), ProgId("OlSentItemsEvents.AddinModule"), RunInstaller(true)]
    public class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule()
        {
            InitializeComponent();
        }
 
        #region Component Designer generated code
        /// <summary>
        /// Required by designer
        /// </summary>
        private System.ComponentModel.IContainer components;
 
        /// <summary>
        /// Required by designer support - do not modify
        /// the following method
        /// </summary>
        private void InitializeComponent()
        {
			// 
			// AddinModule
			// 
			this.AddinName = "Outlook Sent Items Events Example";
			this.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook;
			this.AddinStartupComplete += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinStartupComplete);
			this.AddinBeginShutdown += new AddinExpress.MSO.ADXEvents_EventHandler(this.AddinModule_AddinBeginShutdown);

		}
        #endregion
 
        #region ADX automatic code
 
        // Required by Add-in Express - do not modify
        // the methods within this region
 
        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }
 
        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }
 
        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }
 
        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

		public Outlook.Application OutlookHost()
		{
			return (Outlook.Application)HostApplication;
		}

		private OlItems_SinkHelper olItems;
		private Outlook.MAPIFolder sentFolder = null;

		private void AddinModule_AddinStartupComplete(object sender, System.EventArgs e)
		{
			Outlook.NameSpace oNameSpace = null;
			try
			{
				olItems = new OlItems_SinkHelper(this);
				oNameSpace = OutlookHost().GetNamespace("MAPI");
				sentFolder = oNameSpace.Session.GetDefaultFolder(Outlook.OlDefaultFolders.olFolderSentMail);
				olItems.ConnectTo(sentFolder.Items);
			}
			finally
			{
				if (oNameSpace != null)
				Marshal.ReleaseComObject(oNameSpace);
			}
		}

		private void AddinModule_AddinBeginShutdown(object sender, System.EventArgs e)
		{
			if (olItems != null) 
				olItems.Dispose();
			if (sentFolder != null) 
				Marshal.ReleaseComObject(sentFolder);
		}
 
	}

	[ComImport(), Guid("00063077-0000-0000-C000-000000000046"), InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
	public interface IFolderItemsEvents
	{
		[DispId(61441)]	void ItemAdd(object Item); 
		[DispId(61442)] void ItemChange(object Item); 
		[DispId(61443)] void ItemRemove(); 
	}

	[ClassInterface(ClassInterfaceType.None)]
	public class OlItems_SinkHelper : IFolderItemsEvents, System.IDisposable 
	{ 
		private bool disposed = false;
		private UCOMIConnectionPointContainer oConnPointContainer;
		private UCOMIConnectionPoint m_oConnectionPoint;
		private int m_Cookie;
		private AddinModule owner;

		public OlItems_SinkHelper(AddinModule connect) 
		{ 
			oConnPointContainer = null;
			m_oConnectionPoint = null;
			m_Cookie = 0;
			this.owner = owner;
		} 

		~OlItems_SinkHelper() 
		{ 
			FreeState(); 
		} 

		public void Dispose() 
		{ 
			FreeState(); 
			GC.SuppressFinalize(this); 
		} 

		private void FreeState() 
		{ 
			if (!disposed) 
			{ 
				RemoveConnection(); 
				owner = null; 
				disposed = true; 
			} 
		} 

		public bool ConnectTo(Outlook.Items items) 
		{
			if (m_Cookie != 0) return false;
			try
			{
				Guid guid = new Guid("{00063077-0000-0000-C000-000000000046}");
				oConnPointContainer = (UCOMIConnectionPointContainer)items;
				oConnPointContainer.FindConnectionPoint(ref guid, out m_oConnectionPoint);
				if (m_oConnectionPoint != null)
				{
					m_oConnectionPoint.Advise(this, out m_Cookie);
				}
			}
			catch
			{
				return false;
			}
			return true;
		} 

		public void RemoveConnection() 
		{ 
			if (m_Cookie != 0 && m_oConnectionPoint != null)
			{
				try
				{
					m_oConnectionPoint.Unadvise(m_Cookie);
				}
				catch
				{
				}
				Marshal.ReleaseComObject(m_oConnectionPoint);
				Marshal.ReleaseComObject(oConnPointContainer);
				m_Cookie = 0;
			}
		} 

		public void ItemAdd(object Item) 
		{ 
			MessageBox.Show("A new Item has just been added!");
			Marshal.ReleaseComObject(Item); 
		} 

		public void ItemChange(object Item) 
		{ 
			Marshal.ReleaseComObject(Item); 
		} 

		public void ItemRemove() 
		{ 
		}
	}
}

